local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:GetAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local SetCVar = SetCVar
local GetCVar = GetCVar
local Sound_GameSystem_RestartSoundSystem = Sound_GameSystem_RestartSoundSystem

local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end

local Sound_ChatSystem_GetNumInputDrivers = Sound_ChatSystem_GetNumInputDrivers
local Sound_ChatSystem_GetInputDriverNameByIndex = Sound_ChatSystem_GetInputDriverNameByIndex
local function get_voice_chat_input_drivers_table()
	local n = Sound_ChatSystem_GetNumInputDrivers()
	local i
	local tb = {}
	for i=0,n-1 do
		local v = Sound_ChatSystem_GetInputDriverNameByIndex(i)
		tb[v]=v
	end
	return tb
end

local voice_chat_mode_table = {L["Push-to-Talk"],L["Voice Activated"]}

local Sound_ChatSystem_GetNumOutputDrivers = Sound_ChatSystem_GetNumOutputDrivers
local Sound_ChatSystem_GetOutputDriverNameByIndex = Sound_ChatSystem_GetOutputDriverNameByIndex

local function get_voice_chat_output_drivers_table()
	local n = Sound_ChatSystem_GetNumOutputDrivers()
	local i
	local tb = {}
	for i=0,n-1 do
		local v = Sound_ChatSystem_GetOutputDriverNameByIndex(i)
		tb[v]=v
	end
	return tb
end

local Voice =
{
	name = L["Voice"],
	type = "group",
	args =
	{
		EnableVoiceChat = 
		{
			order = get_order(),
			name = L["Enable"],
			desc = L["Voice Chat is not monitored by Blizzard Entertainment, and your gameplay experience may change based on interaction with other players."],
			type = "toggle",
			set = function(info,val)
				ControlPanel:SetCVar("EnableVoiceChat",val)
			end,
			get = function(info)
				return ControlPanel:GetCVarBool("EnableVoiceChat")
			end,
		},
		Talking =
		{
			order = get_order(),
			name = L["Talking"],
			type = "group",
			args =
			{
				EnableMicrophone = 
				{
					order = get_order(),
					name = L["Enable Microphone"],
					type = "toggle",
					set = function(info,val)
						ControlPanel:SetCVar("EnableMicrophone",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarBool("EnableMicrophone")
					end,
					width = "full"
				},
				OutboundChatVolume = 
				{
					order = get_order(),
					name = L["Microphone Volume"],
					desc = L["Adjusts the volume of your voice"],
					type = "range",
					min = 0,
					max = 2,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("OutboundChatVolume",val*2.5)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("OutboundChatVolume")/2.5
					end,
					width = "full"
				}
			}
		},
		VoiceChatMode =
		{
			order = get_order(),
			name = L["Voice Chat Mode"],
			desc = L["Allows for the transmission of voice chat by just speaking."],
			type = "group",
			childGroups = "tab",
			args = 
			{
				VoiceChatMode =
				{
					order = get_order(),
					name = L["Voice Chat Mode"],
					type = "select",
					values = voice_chat_mode_table,
					set = function(info,val)
						ControlPanel:SetCVar("VoiceChatMode",val-1)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("VoiceChatMode")+1
					end,
					width = "full",
				},
				Push_to_Talk =
				{
					order = get_order(),
					name = L["Push-to-Talk"],
					type = "group",
					args =
					{
						PushToTalkSound =
						{
							order = get_order(),
							name = L["Push-to-Talk Sound"],
							type = "toggle",
							set = function(info,val)
								ControlPanel:SetCVar("PushToTalkSound",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarBool("PushToTalkSound")
							end,
							width = "full"
						}
					}
				},
				VoiceActivated =
				{
					order = get_order(),
					name = L["Voice Activated"],
					type = "group",
					args =
					{
						VoiceActivationSensitivity =
						{
							order = get_order(),
							name = L["Activation Sensitivity"],
							desc = L["Adjusts the sensitivity of the microphone."],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("VoiceActivationSensitivity",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("VoiceActivationSensitivity")
							end,
							width = "full"
						}
					}
				}
			}
		},
		Listening =
		{
			order = get_order(),
			name = L["Listening"],
			type = "group",
			args =
			{
				InboundChatVolume = 
				{
					order = get_order(),
					name = L["Speaker Volume"],
					type = "range",
					min = 0,
					max = 1,
					isPercent = true,
					set = function(info,val)
						ControlPanel:SetCVar("InboundChatVolume",val)
					end,
					get = function(info)
						return ControlPanel:GetCVarNumber("InboundChatVolume")
					end,
					width = "full",
				},
				GameAudioFade=
				{
					order = get_order(),
					name = L["Game Audio Fade"],
					desc = L["Select the level game sounds should adjust to while listening or talking in voice chat."],
					type = "group",
					args =
					{
						ChatSoundVolume =
						{
							order = get_order(),
							name = L["Sound"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("ChatSoundVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("ChatSoundVolume")
							end,
						},
						ChatMusicVolume =
						{
							order = get_order(),
							name = L["Music"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("ChatMusicVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("ChatMusicVolume")
							end,
						},
						ChatAmbienceVolume =
						{
							order = get_order(),
							name = L["Ambience"],
							type = "range",
							min = 0,
							max = 1,
							isPercent = true,
							set = function(info,val)
								ControlPanel:SetCVar("ChatAmbienceVolume",val)
							end,
							get = function(info)
								return ControlPanel:GetCVarNumber("ChatAmbienceVolume")
							end,
						},
					}
				},
			}
		},

		Hardware =
		{
			order = get_order(),
			name = L["Hardware"],
			desc = L["Requires a restart of the game's sound systems"].."\n\n"..L["ControlPanel won't manage these options."],
			type = "group",
			args =
			{
				Sound_VoiceChatInputDriverName =
				{
					order = get_order(),
					name = L["Microphone"],
					type = "select",
					values = get_voice_chat_input_drivers_table,
					set = function(info,val)
						SetCVar("Sound_VoiceChatInputDriverName",val)
					end,
					get = function(info)
						return GetCVar("Sound_VoiceChatInputDriverName")
					end,
					width = "full",
				},
				Sound_VoiceChatOutputDriverName =
				{
					order = get_order(),
					name = L["Speaker"],
					type = "select",
					values = get_voice_chat_output_drivers_table,
					set = function(info,val)
						SetCVar("Sound_VoiceChatOutputDriverName",val)
					end,
					get = function(info)
						return GetCVar("Sound_VoiceChatOutputDriverName")
					end,
					width = "full",
				},
				restart_sound_systems =
				{
					order = get_order(),
					name = L["Restart sound systems"],
					type = "execute",
					func = function(info)
						Sound_GameSystem_RestartSoundSystem()
					end,
					width = "full"
				}
			}
		}
	}
}

ControlPanel_Options:push("Voice",Voice)